//
//  Ride.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "Ride.h"
#import <AddressBookUI/AddressBookUI.h>
#import "TaxiLocationDelegate.h"
#import "TaxiAPIClient.h"

@implementation Ride

- (id)initWithAttributes:(NSDictionary *)attributes
{
    self = [self init];
    
    if (self){
        [self setValue:attributes[@"locator"] forKey:@"locator"];
        [self setValue:attributes[@"bookingTime"] forKey:@"bookingTime"];
        [self setValue:attributes[@"latitude"] forKey:@"latitude"];
        [self setValue:attributes[@"longitude"] forKey:@"longitude"];
        self.status = [[self class] statusFromString:attributes[@"status"]];
        [self updateAddress];
        [[TaxiLocationDelegate sharedLocationDelegate] addObserver:self forKeyPath:@"lastLocation" options:NSKeyValueObservingOptionNew context:nil];
        [self updateDistanceFromLocation:[TaxiLocationDelegate sharedLocationDelegate].lastLocation];
    }
    
    return self;
}

- (void)dealloc
{
    [[TaxiLocationDelegate sharedLocationDelegate] removeObserver:self forKeyPath:@"lastLocation"];
}

#pragma mark - Key-Value Observing

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if ([keyPath isEqual:@"lastLocation"]) {
        CLLocation *newLocation = change[NSKeyValueChangeNewKey];

        [self updateDistanceFromLocation:newLocation];
    }
}

#pragma mark - MKAnnotation

- (NSString *)title
{
    return NSLocalizedString(@"Cliente", @"Etiqueta cliente en mapa");
}

- (NSString *)subtitle
{
    return self.address;
}

- (CLLocationCoordinate2D)coordinate
{
    CLLocationDegrees lat = [self.latitude doubleValue];
    CLLocationDegrees lon = [self.longitude doubleValue];
    CLLocationCoordinate2D coord = CLLocationCoordinate2DMake(lat, lon);
    if (NO == CLLocationCoordinate2DIsValid(coord))
        NSLog(@"Invalid Centroid: lat=%lf lon=%lf", lat, lon);
    return coord;
}

#pragma mark - Actualización campos auxiliares

// Hace un geocoding inverso para obtener la dirección a partir de la coordenada
- (void)updateAddress
{
    CLGeocoder *geocoder = [[CLGeocoder alloc] init];
    CLLocation *location = [[CLLocation alloc] initWithLatitude:self.latitude.doubleValue longitude:self.longitude.doubleValue];
    [geocoder reverseGeocodeLocation:location completionHandler:^(NSArray *placemarks, NSError *error) {
        if (placemarks && placemarks.count > 0) {
            MKPlacemark *placemark = placemarks[0];
            
            CLLocationCoordinate2D coordinate = CLLocationCoordinate2DMake(self.latitude.doubleValue, self.longitude.doubleValue);
            self.placemark = [[MKPlacemark alloc] initWithCoordinate:coordinate addressDictionary:placemark.addressDictionary];
            [self willChangeValueForKey:@"subtitle"];
            self.address = ABCreateStringWithAddressDictionary(self.placemark.addressDictionary, NO);
            [self didChangeValueForKey:@"subtitle"];
        }
    }];
}

// Actualiza distancia del servicio solicitado a la ubicación indicada
- (void)updateDistanceFromLocation:(CLLocation *)location
{
    if (location) {
        CLLocation *destinationLocation = [[CLLocation alloc] initWithLatitude:self.coordinate.latitude longitude:self.coordinate.longitude];
        self.distance = [location distanceFromLocation:destinationLocation];        
    }
}

#pragma mark - Interfaz con API

// Inicia servicio tras llegar hasta el cliente
- (void)beginRideWithSuccess:(void (^)(void))success failure:(void (^)(NSError *error))failure
{
    [[TaxiAPIClient sharedClient] beginRideWithSuccess:^(id JSON) {
        self.status = TaxiRideStatusEnRoute;
        if (success) {
            success();
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

// Finaliza servicio
- (void)endRideWithSuccess:(void (^)(void))success failure:(void (^)(NSError *error))failure
{
    [[TaxiAPIClient sharedClient] endRideWithSuccess:^(id JSON) {
        self.status = TaxiRideStatusComplete;
        if (success) {
            success();
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

// Puntúa servicio
- (void)rateRideWithRating:(NSNumber *)rating success:(void (^)(void))success failure:(void (^)(NSError *error))failure
{
    [[TaxiAPIClient sharedClient] rateRideWithLocator:self.locator rating:rating success:^(id JSON) {
        self.status = TaxiRideStatusRated;
        if (success) {
            success();
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}


// Obtiene servicio con localizador indicado
+ (void)activeRideWithLocator:(NSString *)locator success:(void (^)(Ride *ride))success failure:(void (^)(NSError *error))failure
{
    [[TaxiAPIClient sharedClient] getRequestedRideWithLocator:locator success:^(id JSON){
        Ride *ride = nil;
        if ([JSON count] > 0) {
            ride = [[Ride alloc] initWithAttributes:JSON];
        }
        if (success) {
            success(ride);
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}

// Obtiene servicio pendiente de puntuar para el dispositivo actual
+ (void)unratedRideWithSuccess:(void (^)(Ride *ride))success failure:(void (^)(NSError *error))failure
{
    [[TaxiAPIClient sharedClient] getUnratedRideWithSuccess:^(id JSON){
        Ride *ride = nil;
        if ([JSON count] > 0) {
            ride = [[Ride alloc] initWithAttributes:JSON];
        }
        if (success) {
            success(ride);
        }
    } failure:^(NSError *error) {
        if (failure) {
            failure(error);
        }
    }];
}


#pragma mark - Campo status

// "Traducción" de los estados de string a enum
+ (NSDictionary *)statusDisplayNames
{
    return @{@(TaxiRideStatusPending) : @"pending",
             @(TaxiRideStatusAssigned) : @"assigned",
             @(TaxiRideStatusAccepted) : @"accepted",
             @(TaxiRideStatusEnRoute) : @"en_route",
             @(TaxiRideStatusComplete) : @"complete",
             @(TaxiRideStatusRated) : @"rated"
             };
}

// De enum a string
- (NSString *)statusDisplayName
{
    return [[self class] statusDisplayNames][@(self.status)];
}

// De string a enum
+ (TaxiRideStatus)statusFromString:(NSString *)status
{
    return [[[[[self class] statusDisplayNames] allKeysForObject:status] lastObject] integerValue];
}

#pragma mark - Formato campo distancia

- (NSString *)distanceDescription
{
    return [[self class] stringWithDistance:self.distance];
}

#define METERS_TO_FEET  3.2808399
#define METERS_TO_MILES 0.000621371192
#define METERS_CUTOFF   1000
#define FEET_CUTOFF     3281
#define FEET_IN_MILES   5280

// Formatea distancia según el string indicado, dependiendo de la configuración del locale del dispositivo
// Si la distancia pasa de un umbral, cambia de unidades (metros -> km, pies -> millas)
+ (NSString *)stringWithDistance:(double)distance
{
    BOOL isMetric = [[[NSLocale currentLocale] objectForKey:NSLocaleUsesMetricSystem] boolValue];
    
    NSString *format;
    
    if (isMetric) {
        if (distance < METERS_CUTOFF) {
            format = NSLocalizedString(@"%@ metros", @"Formato String para metros");
        } else {
            format = NSLocalizedString(@"%@ km", @"Formato String para kilometros");
            distance = distance / 1000;
        }
    } else { // se supone Imperial / U.S.
        distance = distance * METERS_TO_FEET;
        if (distance < FEET_CUTOFF) {
            format = NSLocalizedString(@"%@ pies", @"Formato String para pies");
        } else {
            format = NSLocalizedString(@"%@ millas", @"Formato String para millas");
            distance = distance / FEET_IN_MILES;
        }
    }
    
    return [NSString stringWithFormat:format, [self stringWithDouble:distance]];
}

// Devuelve un string con un decimal y usando comas y puntos según el locale del dispositivo
+ (NSString *)stringWithDouble:(double)value
{
    NSNumberFormatter *numberFormatter = [[NSNumberFormatter alloc] init];
    [numberFormatter setLocale:[NSLocale currentLocale]];
    [numberFormatter setNumberStyle:NSNumberFormatterDecimalStyle];
    [numberFormatter setMaximumFractionDigits:1];
    return [numberFormatter stringFromNumber:[NSNumber numberWithDouble:value]];
}

@end
