//
//  TaxiAPIClient.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "TaxiAPIClient.h"
#import <AFNetworking.h>
#import <LUKeychainAccess.h>

#ifdef DEBUG
static NSString * const kTaxiAPIBaseURLString = @"http://taxi.dev/api/v1/";
#else
static NSString * const kTaxiAPIBaseURLString = @"http://taxi.rogel.me/api/v1/";
#endif

#if TARGET_IPHONE_SIMULATOR
static NSString * const kDeviceSimulator = @"DEVICE_ID_SIMULATOR";
#endif

@implementation TaxiAPIClient

// Singleton para que sólo haya una instancia
+ (TaxiAPIClient *)sharedClient {
    static TaxiAPIClient *_sharedClient = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _sharedClient = [[self alloc] initWithBaseURL:[NSURL URLWithString:kTaxiAPIBaseURLString]];
    });
    
    return _sharedClient;
}

- (id)initWithBaseURL:(NSURL *)url {
    self = [super initWithBaseURL:url];
    if (!self) {
        return nil;
    }
    
    // Configura cliente con url base, JSON, y que no rechace respuestas con mime text/html
    [self registerHTTPOperationClass:[AFJSONRequestOperation class]];
    [AFJSONRequestOperation addAcceptableContentTypes:[NSSet setWithObject:@"text/html"]];

	[self setDefaultHeader:@"Accept" value:@"application/json"];

    return self;
}

#pragma mark - API para cliente

// Obtiene taxis disponibles
// get '/api/v1/available_taxis'
- (void)getAvailableTaxisWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    [self performOperationWithMethod:@"GET" path:@"available_taxis" parameters:nil success:success failure:failure];
}

// Solicita taxi en un lugar indicado
// post '/api/v1/hail_taxi', { device: DEVICE_ID, latitude: DEVICE_LAT, longitude: DEVICE_LONG }
- (void)hailTaxiWithCoordinate:(CLLocationCoordinate2D)coordinate success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *device = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"device"];
    
#if TARGET_IPHONE_SIMULATOR
    device = kDeviceSimulator;
#endif

    if (device) {
        [self performOperationWithMethod:@"POST" path:@"hail_taxi" parameters:@{@"device": device, @"latitude": @(coordinate.latitude), @"longitude": @(coordinate.longitude)} success:success failure:failure];
    }
}

// Reserva taxi a una hora y lugar indicados
// post '/api/v1/schedule_pickup', { device: DEVICE_ID, latitude: DEVICE_LAT, longitude: DEVICE_LONG, booking_time: @time }
- (void)schedulePickupWithCoordinate:(CLLocationCoordinate2D)coordinate atTime:(NSDate *)time success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *device = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"device"];
    NSDateFormatter* dateFormatter = [[NSDateFormatter alloc] init];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneWithAbbreviation:@"UTC"]];
    [dateFormatter setDateFormat:@"yyyy-MM-dd HH:mm:ss 'UTC'"];
    [dateFormatter setTimeZone:[NSTimeZone timeZoneForSecondsFromGMT:0]];

    NSString *timeString = [dateFormatter stringFromDate:time];
    
#if TARGET_IPHONE_SIMULATOR
    device = kDeviceSimulator;
#endif

    if (device) {
        [self performOperationWithMethod:@"POST" path:@"schedule_pickup" parameters:@{@"device": device, @"booking_time": timeString, @"latitude": @(coordinate.latitude), @"longitude": @(coordinate.longitude)} success:success failure:failure];
    }
}

// Obtiene taxi asignado para un localizador
// get '/api/v1/assigned_taxi', { device: DEVICE_ID, locator: LOCATOR }
- (void)getAssignedTaxiWithLocator:(NSString *)locator success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *device = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"device"];

#if TARGET_IPHONE_SIMULATOR
    device = kDeviceSimulator;
#endif

    if (device) {
        [self performOperationWithMethod:@"GET" path:@"assigned_taxi" parameters:@{@"device": device, @"locator" : locator} success:success failure:failure];
    }
}

// Obtiene servicio solicitado para un localizador
// get '/api/v1/requested_ride', { device: DEVICE_ID, locator: LOCATOR }
- (void)getRequestedRideWithLocator:(NSString *)locator success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *device = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"device"];

#if TARGET_IPHONE_SIMULATOR
    device = kDeviceSimulator;
#endif

    if (device) {
        [self performOperationWithMethod:@"GET" path:@"requested_ride" parameters:@{@"device": device, @"locator" : locator} success:success failure:failure];
    }
}

// Obtiene servicio pendiente de puntuar
// get '/api/v1/unrated', { device: DEVICE_ID }
- (void)getUnratedRideWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *device = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"device"];
    
#if TARGET_IPHONE_SIMULATOR
    device = kDeviceSimulator;
#endif

    if (device) {
        [self performOperationWithMethod:@"GET" path:@"unrated" parameters:@{@"device": device} success:success failure:failure];
    }
}

// Puntua servicio para un localizador
// post '/api/v1/rate', { device: DEVICE_ID, locator: LOCATOR, rating: RATING }
- (void)rateRideWithLocator:(NSString *)locator rating:(NSNumber *)rating success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *device = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"device"];
    
#if TARGET_IPHONE_SIMULATOR
    device = kDeviceSimulator;
#endif

    if (device) {
        [self performOperationWithMethod:@"POST" path:@"rate" parameters:@{@"device": device, @"locator": locator, @"rating": rating} success:success failure:failure];
    }
}


#pragma mark - API para taxista

// Hace login
// post '/api/v1/tokens', { username: @driver.username, password: @driver.password }
- (void)loginWithUsername:(NSString *)username password:(NSString *)password success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    [self performOperationWithMethod:@"POST" path:@"tokens" parameters:@{@"username": username, @"password": password} success:success failure:failure];
}

// Hace logout
// delete "/api/v1/tokens/#{@token}"
- (void)logoutWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    NSString *logoutUrl = [NSString stringWithFormat:@"tokens/%@.json", token];

    [self performOperationWithMethod:@"DELETE" path:logoutUrl parameters:nil success:success failure:failure];
}

// Actualiza identificador del dispositivo del taxista
// post '/api/v1/update_device_id', { auth_token: @token, device: TAXI_DEVICE_ID }
- (void)updateTaxiDeviceWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    NSString *device = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"device"];

    if (!device) device = @""; // Para el caso de borrar el identificador

    if (token) {
#if TARGET_IPHONE_SIMULATOR
        device = kDeviceSimulator;
#endif
        if (device) {
            [self performOperationWithMethod:@"POST" path:@"update_device_id" parameters:@{@"auth_token": token, @"device": device} success:success failure:failure];
        }
    }
}

// Actualiza disponibilidad del taxi
// post '/api/v1/update_availability', { auth_token: @token, available: available }
- (void)updateTaxiAvailabilityWithStatus:(BOOL)available success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"POST" path:@"update_availability" parameters:@{@"auth_token": token, @"available": @(available)} success:success failure:failure];
}

// Actualiza posición del taxi
// post '/api/v1/update_position', { auth_token: @token, latitude: TAXI_DEVICE_LAT, longitude: TAXI_DEVICE_LONG }
- (void)updateTaxiPositionWithCoordinate:(CLLocationCoordinate2D)coordinate success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"POST" path:@"update_position" parameters:@{@"auth_token": token, @"latitude": @(coordinate.latitude), @"longitude": @(coordinate.longitude)} success:success failure:failure];
}

// Obtiene información de la asignación
// get '/api/v1/assignment_information', { auth_token: @token }
- (void)getAssignmentInformationWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"GET" path:@"assignment_information" parameters:@{@"auth_token": token} success:success failure:failure];
}

// Obtiene reputación del taxista
// get '/api/v1/reputation', { auth_token: @token }
- (void)getReputationWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"GET" path:@"reputation" parameters:@{@"auth_token": token} success:success failure:failure];
}

// Obtiene información del servicio a realizar
// get '/api/v1/ride_information', { auth_token: @token }
- (void)getRideInformationWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"GET" path:@"ride_information" parameters:@{@"auth_token": token} success:success failure:failure];
}

// Obtiene información del taxi
// get '/api/v1/taxi_information', { auth_token: @token }
- (void)getTaxiInformationWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"GET" path:@"taxi_information" parameters:@{@"auth_token": token} success:success failure:failure];
}

// Acepta una asignación
// post '/api/v1/accept_ride', { auth_token: @token }
- (void)acceptRideWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"POST" path:@"accept_ride" parameters:@{@"auth_token": token} success:success failure:failure];
}

// Rechaza una asignación
// post '/api/v1/reject_ride', { auth_token: @token }
- (void)rejectRideWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"POST" path:@"reject_ride" parameters:@{@"auth_token": token} success:success failure:failure];
}

// Inicia un servicio tras llegar al cliente
// post '/api/v1/begin_ride', { auth_token: @token }
- (void)beginRideWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"POST" path:@"begin_ride" parameters:@{@"auth_token": token} success:success failure:failure];
}

// Finaliza un servicio
// post '/api/v1/end_ride', { auth_token: @token }
- (void)endRideWithSuccess:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSString *token = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"];
    
    [self performOperationWithMethod:@"POST" path:@"end_ride" parameters:@{@"auth_token": token} success:success failure:failure];
}


#pragma mark - Métodos privados

// Método para interacción con servidor
- (void)performOperationWithMethod:(NSString *)method path:(NSString *)path parameters:(NSDictionary *)parameters success:(void (^)(id))success failure:(void (^)(NSError *))failure
{
    NSURLRequest *request = [self requestWithMethod:method path:path parameters:parameters];
    
    AFJSONRequestOperation *operation = [AFJSONRequestOperation JSONRequestOperationWithRequest:request success:^(NSURLRequest *request, NSHTTPURLResponse *response, id JSON) {
        if (success) {
            success(JSON);            
        }
    } failure:^(NSURLRequest *request, NSHTTPURLResponse *response, NSError *error, id JSON) {
        if (failure) {
            failure(error);
        }
    }];
    
    [operation start];
}


@end
