//
//  TaxiLoginViewController.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "TaxiLoginViewController.h"
#import <SVProgressHUD.h>
#import <LUKeychainAccess.h>
#import <UINavigationBar+FlatUI.h>
#import <UIColor+FlatUI.h>
#import <UIFont+FlatUI.h>
#import <FUIButton.h>
#import "TaxiAPIClient.h"
#import "Taxi.h"
#import "TaxiLoggedInSegue.h"


@interface TaxiLoginViewController () <UIGestureRecognizerDelegate, UITextFieldDelegate>
@property (weak, nonatomic) IBOutlet UITextField *loginField;
@property (weak, nonatomic) IBOutlet UITextField *passwordField;
@property (weak, nonatomic) IBOutlet FUIButton *loginButton;
@end

@implementation TaxiLoginViewController

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];

    // Si ya se había hecho login previamente, hacer transición sin animación
    if ([[LUKeychainAccess standardKeychainAccess] stringForKey:@"authtoken"]) {
        [self performSegueWithIdentifier:@"Home" sender:self];
    }
}

- (IBAction)doLogin
{
    NSString *login = self.loginField.text;
    NSString *password = self.passwordField.text;

    [SVProgressHUD showWithStatus:NSLocalizedString(@"Autenticando...", @"Mensaje de autenticación en progreso") maskType:SVProgressHUDMaskTypeBlack];

    [[TaxiAPIClient sharedClient] loginWithUsername:login password:password success:^(id JSON) {
        NSString *token = [JSON objectForKey:@"token"];
        NSLog(@"%@", token);
        
        // Guardar token
        [[LUKeychainAccess standardKeychainAccess] setString:token forKey:@"authtoken"];
        [[LUKeychainAccess standardKeychainAccess] setString:login forKey:@"username"];
        [Taxi updateDeviceToken];

        // Hacer transición con animación
        [SVProgressHUD showSuccessWithStatus:NSLocalizedString(@"Bienvenido", @"Mensaje de bienvenida tras login")];
        [self performSegueWithIdentifier:@"Home" sender:self];
    } failure:^(NSError *error) {
        [SVProgressHUD showErrorWithStatus:NSLocalizedString(@"Usuario o contraseña incorrecto", @"Mensaje de error tras login incorrecto")];
    }];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    if ([[segue identifier] isEqualToString:@"Home"]) {
        // Configura animación de la transición
        TaxiLoggedInSegue *loginSegue = (TaxiLoggedInSegue *)segue;

        [loginSegue setAnimated:![self.loginField.text isEqualToString:@""]];
    }
}

# pragma mark - Interacción con campos y teclado

- (void)viewDidLoad
{
    [super viewDidLoad];

    self.view.backgroundColor = [UIColor cloudsColor];

    // Configurar Flat UI
    self.navigationController.navigationBar.titleTextAttributes = @{UITextAttributeFont: [UIFont boldFlatFontOfSize:0.0]};
    [self.navigationController.navigationBar configureFlatNavigationBarWithColor:[UIColor midnightBlueColor]];

    self.loginField.font = [UIFont boldFlatFontOfSize:16];
    self.passwordField.font = [UIFont boldFlatFontOfSize:16];

    self.loginButton.buttonColor = [UIColor peterRiverColor];
    self.loginButton.shadowColor = [UIColor belizeHoleColor];
    self.loginButton.shadowHeight = 3.0f;
    self.loginButton.cornerRadius = 6.0f;
    self.loginButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.loginButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.loginButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];

    // Añade gesto "pulsar" sobre la vista (para cerrar el teclado)
    UITapGestureRecognizer *tgr = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(viewTapped:)];
    tgr.delegate = self;
    [self.view addGestureRecognizer:tgr];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    self.passwordField.text = @"";
    
    // Observar cuándo se muestre el teclado
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(keyboardDidShow:)
                                                 name:UIKeyboardDidShowNotification
                                               object:nil];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    
    // Dejar de observar
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewTapped:(UITapGestureRecognizer *)tgr
{
    // Quita el teclado
    [self.view endEditing:NO];
}

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
{
    // Detectar si se ha tocado en un campo de texto o no
    if ([touch.view isKindOfClass:[UITextField class]]) {
        return NO;
    }
    return YES;
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    // Se ha pulsado en el botón "Return" del teclado
    if ([textField isEqual:self.loginField]) {
        // Si estábamos en el campo de usuario pasar al de contraseña
        [self.passwordField becomeFirstResponder];
    } else if ([textField isEqual:self.passwordField]) {
        // Si estábamos en el campo de contraseña hacer login
        [textField resignFirstResponder];
        [self doLogin];
    }
    return YES;
}

-(void)keyboardDidShow:(NSNotification *)aNotification
{
    // Se ha mostrado el teclado, hacer scroll para que se vean los campos de texto y el botón de login
    NSInteger sections = [self.tableView numberOfSections];
    NSInteger rows = [self.tableView numberOfRowsInSection:sections-1];
    CGRect frame = [self.tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:(rows - 1) inSection:(sections - 1)]].frame;
    frame.size.height += 90;
    [self.tableView scrollRectToVisible:frame animated:YES];
}

@end
