//
//  TaxiMainViewController.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "TaxiMainViewController.h"
#import "TaxiAPIClient.h"
#import "TaxiLocationDelegate.h"
#import <AddressBookUI/AddressBookUI.h>
#import <LUKeychainAccess.h>
#import <AFNetworking.h>
#import <SVProgressHUD.h>
#import "Taxi.h"
#import "Assignment.h"
#import <MapKit/MapKit.h>
#import <UIColor+FlatUI.h>
#import <UIFont+FlatUI.h>
#import <UIBarButtonItem+FlatUI.h>
#import <UINavigationBar+FlatUI.h>
#import "TaxiSegmentedControl.h"
#import "TaxiAssignmentViewController.h"
#import "TaxiRideViewController.h"
#import <FUIButton.h>


@interface TaxiMainViewController () <CLLocationManagerDelegate>
@property (strong, nonatomic) IBOutletCollection(UILabel) NSArray *allLabels;
@property (weak, nonatomic) IBOutlet TaxiSegmentedControl *availabilityControl;
@property (weak, nonatomic) IBOutlet MKMapView *positionMapView;
@property (weak, nonatomic) IBOutlet UILabel *addressLabel;
@property (weak, nonatomic) IBOutlet FUIButton *reputationButton;
- (IBAction)doLogout:(id)sender;
- (IBAction)updateAvailability:(UISegmentedControl *)sender;
@property (nonatomic, assign) BOOL available;
@property (nonatomic, strong) Assignment *currentAssignment;
@end

@implementation TaxiMainViewController

- (void)viewDidLoad
{
    [super viewDidLoad];

    NSString *username = [[LUKeychainAccess standardKeychainAccess] stringForKey:@"username"];

    self.navigationItem.title = [NSString stringWithFormat:NSLocalizedString(@"Hola %@", @"Título pantalla taxista identificado"), username];
    
    self.view.backgroundColor = [UIColor cloudsColor];

    // Configurar Flat UI
    [self.allLabels makeObjectsPerformSelector:@selector(setFont:) withObject:[UIFont flatFontOfSize:16]];
    self.addressLabel.text = @"";

    self.availabilityControl.selectedFont = [UIFont boldFlatFontOfSize:16];
    self.availabilityControl.selectedFontColor = [UIColor cloudsColor];
    self.availabilityControl.deselectedFont = [UIFont flatFontOfSize:16];
    self.availabilityControl.deselectedFontColor = [UIColor cloudsColor];
    self.availabilityControl.selectedColor = [UIColor emerlandColor];
    self.availabilityControl.deselectedColor = [UIColor asbestosColor];
    self.availabilityControl.dividerColor = [UIColor midnightBlueColor];
    self.availabilityControl.cornerRadius = 5.0;

    [self.availabilityControl setTitle:NSLocalizedString(@"Disponible", @"Etiqueta control segmentado - disponible") forSegmentAtIndex:0];
    [self.availabilityControl setTitle:NSLocalizedString(@"Ocupado", @"Etiqueta control segmentado - ocupado") forSegmentAtIndex:1];

    self.reputationButton.buttonColor = [UIColor peterRiverColor];
    self.reputationButton.shadowColor = [UIColor belizeHoleColor];
    self.reputationButton.shadowHeight = 3.0f;
    self.reputationButton.cornerRadius = 6.0f;
    self.reputationButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.reputationButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.reputationButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];


    [UIBarButtonItem configureFlatButtonsWithColor:[UIColor peterRiverColor]
                                  highlightedColor:[UIColor belizeHoleColor]
                                      cornerRadius:3];
    
    [self.navigationItem setHidesBackButton:YES];
    [self.navigationItem.rightBarButtonItem removeTitleShadow];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    [[TaxiLocationDelegate sharedLocationDelegate] startUpdatingLocation];

    // Actualiza mapa continuamente con posición usuario
    self.positionMapView.userTrackingMode = MKUserTrackingModeFollow;

    // Deshabilita control de disponibilidad hasta tener respuesta del servidor
    self.availabilityControl.enabled = NO;
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];

    // Actualizar información y observar eventos de cambio de posición y de solicitud de actualización de datos
    [self performSelector:@selector(updateData) withObject:nil afterDelay:0];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didChangeLocation:) name:@"DID_CHANGE_LOCATION" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(updateData) name:@"UPDATE_DATA" object:nil];
}

- (void)viewDidDisappear:(BOOL)animated
{
    // Dejar de observar eventos
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"DID_CHANGE_LOCATION" object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"UPDATE_DATA" object:nil];
}

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    // Detectar y configurar transición a asignación y a servicio
    if ([[segue identifier] isEqualToString:@"Assignment"]) {
        TaxiAssignmentViewController *vc = [segue destinationViewController];
        vc.currentAssignment = self.currentAssignment;
    } else if ([[segue identifier] isEqualToString:@"Ride"]) {
        TaxiRideViewController *vc = [segue destinationViewController];
        vc.currentRide = self.currentAssignment.ride;
    }
}

# pragma mark - Notificaciones

// Ejecutado al mostrar la pantalla y al recibir notificación
- (void)updateData
{
    // Obtiene información del taxi y configura controles en pantalla con los valores indicados
    [Taxi fetchWithSuccess:^(Taxi *taxi){
        [self.availabilityControl setSelectedSegmentIndex:(taxi.available.boolValue ? 0 : 1)];
        self.availabilityControl.selectedColor = taxi.available.boolValue ? [UIColor emerlandColor] : [UIColor pomegranateColor];
        self.availabilityControl.enabled = YES;

        self.available = [taxi.available boolValue];
    } failure:^(NSError *error){
        [self logout];
    }];

    // Comprobar si tenemos algún viaje asignado (pendiente de respuesta o aceptado)
    [Assignment currentAssignmentWithSuccess:^(Assignment *assignment){
        self.currentAssignment = assignment;
        if (assignment) {
            if (assignment.status == TaxiAssignmentStatusPending) {
                // Hay uno pendiente de respuesta, mostrar pantalla
                [self performSegueWithIdentifier:@"Assignment" sender:assignment];
            } else if (assignment.status == TaxiAssignmentStatusAccepted) {
                // Hay uno aceptado, mostrar información del viaje
                [self performSegueWithIdentifier:@"Ride" sender:assignment.ride];
            }
        }
    } failure:^(NSError *error){
        [self logout];
    }];
}

// Se ha cambiado la posición del usuario
- (void)didChangeLocation:(NSNotification *)notification
{
    NSDictionary *params = notification.userInfo;
    CLLocation *location = params[@"location"];

    if (location) {
        CLGeocoder *geocoder = [[CLGeocoder alloc] init];

        // Obtener dirección para la nueva coordenada
        [geocoder reverseGeocodeLocation:location completionHandler:^(NSArray *placemarks, NSError *error) {
            if (placemarks && placemarks.count > 0) {
                CLPlacemark *placemark = [placemarks objectAtIndex:0];
                self.addressLabel.text = ABCreateStringWithAddressDictionary(placemark.addressDictionary, NO);
            }
        }];
        
        // Actualizar posicion
        [Taxi didChangeLocation:location];
        [self.positionMapView setRegion:MKCoordinateRegionMake(location.coordinate, MKCoordinateSpanMake(0.005, 0.005)) animated:YES];
    }
}

#pragma mark - Interacción con usuario

// Actualizar disponibilidad
- (IBAction)updateAvailability:(UISegmentedControl *)sender
{
    BOOL available = (sender.selectedSegmentIndex == 0);

    self.available = available;

    if (available) {
        [[TaxiLocationDelegate sharedLocationDelegate] startUpdatingLocation];
    } else {
        [[TaxiLocationDelegate sharedLocationDelegate] stopUpdatingLocation];
    }
    self.availabilityControl.selectedColor = available ? [UIColor emerlandColor] : [UIColor pomegranateColor];
    
    [[TaxiAPIClient sharedClient] updateTaxiAvailabilityWithStatus:available success:^(id JSON) {
        [SVProgressHUD showSuccessWithStatus:NSLocalizedString(@"Disponibilidad actualizada", @"Mensaje aviso tras cambiar disponibilidad")];
    } failure:^(NSError *error) {
        [SVProgressHUD dismiss];
        [self logout];
    }];
}

// Botón cerrar sesión
- (IBAction)doLogout:(id)sender
{
    [SVProgressHUD showSuccessWithStatus:NSLocalizedString(@"Sesión cerrada", @"Mensaje aviso tras cerrar sesión")];

    [self logout];
}

// Realiza cierre de sesión, borra identificador dispositivo y vuelve a pantalla login
- (void)logout
{
    [[LUKeychainAccess standardKeychainAccess] setData:nil forKey:@"device"];
    [Taxi updateDeviceToken];

    [[LUKeychainAccess standardKeychainAccess] setData:nil forKey:@"authtoken"];
    [[LUKeychainAccess standardKeychainAccess] setData:nil forKey:@"username"];

    [[TaxiAPIClient sharedClient] logoutWithSuccess:nil failure:nil];

    [self.navigationController performSelector:@selector(popToRootViewControllerAnimated:) withObject:@YES afterDelay:0];
}

@end
