//
//  TaxiRideViewController.m
//  Taxi
//
//  Copyright (c) 2013 Toni Rogel. All rights reserved.
//

#import "TaxiRideViewController.h"
#import <FUIButton.h>
#import <UIColor+FlatUI.h>
#import <UIFont+FlatUI.h>
#import "TaxiLocationDelegate.h"
#import <SVProgressHUD.h>

@interface TaxiRideViewController ()

@property (strong, nonatomic) IBOutletCollection(UILabel) NSArray *allLabels;
@property (weak, nonatomic) IBOutlet FUIButton *routeButton;
@property (weak, nonatomic) IBOutlet FUIButton *finishButton;
@property (weak, nonatomic) IBOutlet MKMapView *positionMapView;
@property (weak, nonatomic) IBOutlet UILabel *addressLabel;
@property (weak, nonatomic) IBOutlet UILabel *distanceLabel;
- (IBAction)showRouteToClient;
- (IBAction)endRide;

@end

@implementation TaxiRideViewController

#pragma mark - Lifecycle

#define MONITORING_RADIUS 200.0
#define MONITORING_IDENTIFIER @"CustomerRegion"

- (void)viewDidLoad
{
    [super viewDidLoad];
    
    self.view.backgroundColor = [UIColor cloudsColor];
    
    // Configurar Flat UI
    [self.allLabels makeObjectsPerformSelector:@selector(setFont:) withObject:[UIFont flatFontOfSize:16]];
    [self.addressLabel setFont:[UIFont boldFlatFontOfSize:24]];
    self.addressLabel.text = @"";
    [self.distanceLabel setFont:[UIFont boldFlatFontOfSize:20]];
    self.distanceLabel.text = self.currentRide.distanceDescription;
    self.distanceLabel.textColor = [UIColor nephritisColor];

    self.routeButton.buttonColor = [UIColor emerlandColor];
    self.routeButton.shadowColor = [UIColor nephritisColor];
    self.routeButton.shadowHeight = 3.0f;
    self.routeButton.cornerRadius = 6.0f;
    self.routeButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.routeButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.routeButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];
    
    self.finishButton.buttonColor = [UIColor alizarinColor];
    self.finishButton.shadowColor = [UIColor pomegranateColor];
    self.finishButton.shadowHeight = 3.0f;
    self.finishButton.cornerRadius = 6.0f;
    self.finishButton.titleLabel.font = [UIFont boldFlatFontOfSize:16];
    [self.finishButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateNormal];
    [self.finishButton setTitleColor:[UIColor cloudsColor] forState:UIControlStateHighlighted];
    
    [self.navigationItem setHidesBackButton:YES];
    [self.positionMapView addAnnotation:self.currentRide];
    
    switch (self.currentRide.status) {
        case TaxiRideStatusAccepted:
            // Activar monitorización de llegada
            [[TaxiLocationDelegate sharedLocationDelegate] startMonitoringRegionWithLocation:self.currentRide.coordinate radius:MONITORING_RADIUS identifier:MONITORING_IDENTIFIER];
            self.routeButton.hidden = NO;
            self.finishButton.hidden = YES;
            break;

        case TaxiRideStatusEnRoute:
            self.routeButton.hidden = YES;
            self.finishButton.hidden = NO;
            break;

        default:
            break;
    }
    self.routeButton.enabled = NO;

}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];

    // Crear map points
    MKMapPoint userPoint = MKMapPointForCoordinate(self.positionMapView.userLocation.coordinate);
    MKMapPoint customerPoint = MKMapPointForCoordinate(self.currentRide.coordinate);
    // Crear map rects con tamaño 0
    MKMapRect userRect = MKMapRectMake(userPoint.x, userPoint.y, 0, 0);
    MKMapRect customerRect = MKMapRectMake(customerPoint.x, customerPoint.y, 0, 0);
    // Crear la unión de ambos rects
    MKMapRect unionRect = MKMapRectUnion(userRect, customerRect);
    MKMapRect unionRectThatFits = [self.positionMapView mapRectThatFits:unionRect edgePadding:UIEdgeInsetsFromString(@"{10.0,10.0,10.0,10.0}")];
    [self.positionMapView setVisibleMapRect:unionRectThatFits animated:YES];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];

    // Observa cambios en la dirección y la distancia del servicio
    [self.currentRide addObserver:self forKeyPath:@"address" options:NSKeyValueObservingOptionNew context:nil];
    [self.currentRide addObserver:self forKeyPath:@"distance" options:NSKeyValueObservingOptionNew context:nil];
    // Observa eventos de entrar en región
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didEnterRegion:) name:@"DID_ENTER_REGION" object:nil];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
    // Deja de observar cambios y eventos
    [self.currentRide removeObserver:self forKeyPath:@"address"];
    [self.currentRide removeObserver:self forKeyPath:@"distance"];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:@"DID_ENTER_REGION" object:nil];
}

// Si se entra en la región monitorizada se inicia el servicio (y se notifica al cliente)
- (void)didEnterRegion:(NSNotification *)notification
{
    NSDictionary *params = notification.userInfo;
    NSString *identifier = params[@"region"];

    if ([identifier isEqualToString:MONITORING_IDENTIFIER]) {
        // Dejar de monitorizar región y notificar al usuario de la llegada
        [[TaxiLocationDelegate sharedLocationDelegate] stopMonitoringRegionWithIdentifier:MONITORING_IDENTIFIER];
        [self.currentRide beginRideWithSuccess:^{
            [self.navigationController popViewControllerAnimated:YES];
        } failure:nil];
    }
}

#pragma mark - Key-Value Observing

// Si hay cambios en dirección o distancia actualiza las etiquetas
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if ([keyPath isEqual:@"address"]) {
        self.addressLabel.text = self.currentRide.address;
        self.routeButton.enabled = YES;
    }
    if ([keyPath isEqual:@"distance"]) {
        self.distanceLabel.text = self.currentRide.distanceDescription;
    }
}

#pragma mark - Acciones

- (IBAction)showRouteToClient
{
    // Abre aplicación de mapas con la ruta hacia el cliente
    MKMapItem *customerMapItem = [[MKMapItem alloc] initWithPlacemark:self.currentRide.placemark];
    customerMapItem.name = self.currentRide.title;
    NSDictionary *options = @{
                              MKLaunchOptionsDirectionsModeKey : MKLaunchOptionsDirectionsModeDriving,
                              };
    [customerMapItem openInMapsWithLaunchOptions:options];
}

- (IBAction)endRide
{
    // Finaliza el servicio (y se notifica al cliente)
    [SVProgressHUD showWithStatus:NSLocalizedString(@"Actualizando...", @"Mensaje de actualización de información en progreso") maskType:SVProgressHUDMaskTypeBlack];
    [self.currentRide endRideWithSuccess:^{
        [self.navigationController popViewControllerAnimated:YES];
        [SVProgressHUD dismiss];
    } failure:nil];
}


@end
