# encoding : utf-8
class Assignment < ActiveRecord::Base
  include Workflow

  # maquina de estados finitos: columna con estado actual
  workflow_column :status

  # relaciones con otros modelos
  belongs_to :taxi
  belongs_to :ride

  attr_accessible :notified_at, :response_received_at, :status, :taxi

  # a ejecutar tras crear un registro
  after_create :notify_taxi_driver
  after_create :assign_to_taxi
  after_create :update_ride

  # maquina de estados finitos: estados, eventos y transiciones permitidas
  workflow do
    state :pending do
      event :accept, transition_to: :accepted
      event :reject, transition_to: :rejected
    end
    state :accepted
    state :rejected
  end

  # filtros usados habitualmente
  scope :accepted, where(status: :accepted)
  scope :rejected, where(status: :rejected)
  scope :for_driver, ->(driver) { where(taxi_id: driver.taxi.id) }

  # aceptar asignacion: se marca el taxi como ocupado y se notifica al cliente
  def accept_assignment
    self.accept!
    self.taxi.update_column(:available, false)
    self.notify_customer
  end

  # rechazar asignacion: se borra la asignacion actual del taxi y se asigna otro taxi
  def reject_assignment
    self.reject!
    self.taxi.update_column(:current_assignment_id, nil)
    self.ride.assign_taxi
  end

  # metodos ejecutados en las transiciones de la maquina de estados finitos
  # al aceptar la asignacion se pasa el evento "aceptar" al servicio
  def accept
    self.ride.accept!
  end

  # al rechazar la asignacion se pasa el evento "rechazar" al servicio
  def reject
    self.ride.reject!
  end

  protected

  # asigna al taxi la asignacion actual
  def assign_to_taxi
    self.taxi.update_column(:current_assignment_id, self)
  end

  # tras crear pasa el evento "asignar" al servicio
  def update_ride
    self.ride.assign!
  end

  # notifica a taxista tras habersele asignado el servicio
  def notify_taxi_driver
    if self.taxi.taxi_driver.device.present?
      Notification.send(self.taxi.taxi_driver.device, Notification::NOTIF_NEW_RIDE, :driver)
    end
    self.update_attribute(:notified_at, Time.now)
  end

  # notifica a cliente tras la aceptacion del servicio por parte del taxista
  def notify_customer
    if self.ride.device.present?
      Notification.send(self.ride.device, Notification::NOTIF_ACCEPTED, :client)
    end
  end

end
