# encoding : utf-8
class Ride < ActiveRecord::Base
  include Workflow

  # maquina de estados finitos: columna con estado actual
  workflow_column :status

  attr_accessible :device, :locator, :booking_time, :latitude, :longitude, :status

  # relaciones con otros modelos
  has_many :assignments, order: :id
  has_one :rating

  # filtros usados habitualmente
  scope :current, -> { where("booking_time <= ?", Time.now) }
  scope :unrated, where(status: :complete)
  scope :pending, where(status: :pending)
  scope :for_device, ->(device) { where(device: device) }
  scope :for_locator, ->(locator) { where(locator: locator) }
  scope :assignment_accepted, joins(:assignments).merge(Assignment.accepted)
  scope :latest, order("booking_time DESC, id DESC")

  # a ejecutar antes de crear un registro
  before_create :set_locator
  before_create :set_booking_time

  # maquina de estados finitos: estados, eventos y transiciones permitidas
  workflow do
    state :pending do
      event :assign, transition_to: :assigned
    end
    state :assigned do
      event :accept, transition_to: :accepted
      event :reject, transition_to: :pending
    end
    state :accepted do
      event :begin, transition_to: :en_route
    end
    state :en_route do
      event :end, transition_to: :complete
    end
    state :complete do
      event :rate, transition_to: :rated
    end
    state :rated
  end

  # asignar taxi:
  # busca los mas cercanos
  # descarta los que ya han rechazado y los que estan ocupados o con asignaciones pendientes
  def assign_taxi
    closest_taxis = Taxi.unassigned.closest_taxis(self.latitude, self.longitude)
    rejected_taxi_ids = self.assignments.rejected.collect(&:taxi_id)
    begin
      taxi = closest_taxis.shift
      taxi = taxi[:taxi] if taxi.present?
    end while taxi.present? and taxi.id.in?(rejected_taxi_ids)
    self.assignments.create(taxi: taxi) if taxi.present?
  end

  def current_assignment
    self.assignments.last
  end

  def accepted_assignment
    self.assignments.accepted.first
  end

  # metodos de ayuda para encontrar el servicio segun dispositivo y localizador
  def self.for_device_and_locator(device, locator)
    for_device(device).for_locator(locator).first
  end

  def self.current_for_device_and_locator(device, locator)
    current.for_device(device).for_locator(locator).first
  end

  def self.current_unrated_for_device(device)
    unrated.current.for_device(device).assignment_accepted.latest.first
  end

  # metodos ejecutados en las transiciones de la maquina de estados finitos
  def begin
    self.notify_arrival
  end

  def end
    self.notify_rate
    self.current_assignment.taxi.update_column(:available, true)
    self.current_assignment.taxi.update_column(:current_assignment_id, nil)
  end

  # puntua servicio
  def rate(value)
    self.create_rating(device: self.device, taxi: self.accepted_assignment.taxi, value: value)
  end

  protected
  # asigna un localizador
  def set_locator
    self.locator = SecureRandom.uuid
  end

  # asigna una hora de recogida, si no hay
  def set_booking_time
    self.booking_time ||= Time.now
  end

  # notifica llegada del taxi al cliente
  def notify_arrival
    if self.device.present?
      Notification.send(self.device, Notification::NOTIF_ARRIVAL, :client)
    end
  end

  # notifica finalizacion del servicio al cliente (para que pueda puntuar)
  def notify_rate
    if self.device.present?
      Notification.send(self.device, Notification::NOTIF_RATE, :client)
    end
  end
end
